import 'dart:async';

import 'package:http/http.dart' as http;
import 'package:ink_runtime/ink.dart';
import 'package:meta/meta.dart';
import 'package:test/test.dart';

typedef TestFunction = void Function(
  Object name,
  dynamic Function() body, {
  String? testOn,
  Timeout? timeout,
  dynamic skip,
  Map<String, dynamic>? onPlatform,
  dynamic tags,
  int? retry,
});

typedef GroupFunction = void Function(Object description, void Function() body,
    {dynamic skip});

abstract class BaseInkTests {
  var client = http.Client();

  Future<String> loadFromUrl(String jsonUrl) {
    var url = Uri.parse(jsonUrl);

    return client.get(url).then((value) => value.body);
  }

  final String spec;
  final Future<Story> Function(String) createStory;

  @protected
  final TestFunction testFunction;

  final TestFunction it;

  final GroupFunction groupFunction;

  BaseInkTests(
      {required this.spec,
      required this.testFunction,
      required this.groupFunction,
      required this.createStory})
      : it = testFunction;

  void run() {
    groupFunction(spec, () {
      tests();

      tearDown(() {
        //story ;
      });
    });
  }

  void tests();
}
