import 'dart:convert';

import 'package:ink_runtime/ink.dart';
import 'package:test/test.dart';

import '../base_ink_tests.dart';
import 'inkfiles/compiled/conditions/all_switch_branches_fail_is_clean.ink.json.dart'
    as all_switch_branches_fail_is_clean;
import 'inkfiles/compiled/conditions/else_branches.ink.json.dart'
    as else_branches;
import 'inkfiles/compiled/conditions/empty_multiline_conditional_branch.ink.json.dart'
    as empty_multiline_conditional_branch;
import 'inkfiles/compiled/conditions/trivial_condition.ink.json.dart'
    as trivial_condition;

class ConditionsSpec extends BaseInkTests {
  ConditionsSpec(Future<Story> Function(String) load,
      {required TestFunction test, required GroupFunction group})
      : super(
          spec: 'Conditions',
          createStory: load,
          groupFunction: group,
          testFunction: test,
        );

  @override
  void tests() {
    it('tests all switch branches fail is clean', () async {
      var story =
          await createStory(jsonEncode(all_switch_branches_fail_is_clean.json));

      story.Continue();
      expect(story.state.evaluationStack.length, 0);
    });

    it('tests else branches', () async {
      var story = await createStory(jsonEncode(else_branches.json));

      expect(story.ContinueMaximally(), 'other\nother\nother\nother\n');
    });

    it('tests empty multiline conditional branch', () async {
      var story = await createStory(
          jsonEncode(empty_multiline_conditional_branch.json));

      expect(story.Continue(), '');
    });

    it('tests trivial condition', () async {
      var story = await createStory(jsonEncode(trivial_condition.json));

      story.Continue();
      expect(story.hasError, false);
    });
  }
}
