import 'dart:convert';

import 'package:ink_runtime/ink.dart';
import 'package:test/test.dart';

import '../base_ink_tests.dart';
import 'inkfiles/compiled/evaluation/arithmetic.ink.json.dart' as arithmetic;
import 'inkfiles/compiled/evaluation/basic_string_literals.ink.json.dart'
    as basic_string_literals;
import 'inkfiles/compiled/evaluation/evaluating_function_variable_state_bug.ink.json.dart'
    as evaluating_function_variable_state_bug;
import 'inkfiles/compiled/evaluation/evaluating_ink_functions_from_game.ink.json.dart'
    as evaluating_ink_functions_from_game;
import 'inkfiles/compiled/evaluation/evaluating_ink_functions_from_game_2.ink.json.dart'
    as evaluating_ink_functions_from_game_2;
import 'inkfiles/compiled/evaluation/evaluation_stack_leaks.ink.json.dart'
    as evaluation_stack_leaks;
import 'inkfiles/compiled/evaluation/factorial_by_reference.ink.json.dart'
    as factorial_by_reference;
import 'inkfiles/compiled/evaluation/factorial_recursive.ink.json.dart'
    as factorial_recursive;
import 'inkfiles/compiled/evaluation/increment.ink.json.dart' as increment;
import 'inkfiles/compiled/evaluation/literal_unary.ink.json.dart'
    as literal_unary;

class EvaluationSpec extends BaseInkTests {
  EvaluationSpec(Future<Story> Function(String) load,
      {required TestFunction test, required GroupFunction group})
      : super(
          spec: 'Evaluation',
          createStory: load,
          groupFunction: group,
          testFunction: test,
        );

  @override
  void tests() {
    it('tests arithmetic', () async {
      var story = await createStory(jsonEncode(arithmetic.json));

      // inkjs can't see 3.0 as a "float", so
      // { 7 / 3.0 } is 2 instead of 2.3333.
      expect(story.ContinueMaximally(), '36\n2\n3\n2\n${7 / 3.0}\n8\n8\n');
    });

    it('tests basic string literal', () async {
      var story = await createStory(jsonEncode(basic_string_literals.json));

      expect(story.ContinueMaximally(), 'Hello world 1\nHello world 2.\n');
    });

    it('tests evaluating function variable state bug', () async {
      var story = await createStory(
          jsonEncode(evaluating_function_variable_state_bug.json));

      expect(story.Continue(), 'Start\n');
      expect(story.Continue(), 'In tunnel.\n');

      var funcResult = story.EvaluateFunction('function_to_evaluate');
      expect(funcResult.returned, 'RIGHT');
      expect(story.Continue(), 'End\n');
    });

    it('tests evaluating ink functions from game', () async {
      var story = await createStory(
          jsonEncode(evaluating_ink_functions_from_game.json));

      story.Continue();

      var returnedDivertTarget = story.EvaluateFunction('test');

      expect(returnedDivertTarget.returned, 'somewhere.here');
    });

    it('tests evaluating ink functions from game 2', () async {
      var story = await createStory(
          jsonEncode(evaluating_ink_functions_from_game_2.json));

      var funcResult = story.EvaluateFunction('func1', []);

      expect(funcResult.output, 'This is a function\n');
      expect(funcResult.returned, 5);

      expect(story.Continue(), 'One\n');

      funcResult = story.EvaluateFunction('func2', []);

      expect(funcResult.output, 'This is a function without a return value\n');
      expect(funcResult.returned, null);

      expect(story.Continue(), 'Two\n');

      funcResult = story.EvaluateFunction('add', [1, 2]);

      expect(funcResult.output, 'x = 1, y = 2\n');
      expect(funcResult.returned, 3);

      expect(story.Continue(), 'Three\n');
    });

    it('tests evaluating stack leaks', () async {
      var story = await createStory(jsonEncode(evaluation_stack_leaks.json));

      expect(story.ContinueMaximally(), 'else\nelse\nhi\n');
      expect(story.state.evaluationStack.length, 0);
    });

    it('tests factorial by reference', () async {
      var story = await createStory(jsonEncode(factorial_by_reference.json));

      expect(story.ContinueMaximally(), '120\n');
    });

    it('tests factorial recursive', () async {
      var story = await createStory(jsonEncode(factorial_recursive.json));

      expect(story.ContinueMaximally(), '120\n');
    });

    it('tests increment', () async {
      var story = await createStory(jsonEncode(increment.json));

      expect(story.ContinueMaximally(), '6\n5\n');
    });

    it('tests literal unary', () async {
      var story = await createStory(jsonEncode(literal_unary.json));

      expect(story.ContinueMaximally(), '-1\nfalse\ntrue\n');
    });
  }
}
