import 'dart:convert';

import 'package:ink_runtime/ink.dart';
import '../../base_ink_tests.dart';
import 'package:test/test.dart';

import '../inkfiles/compiled/inkjs/tests.ink.json.dart' as $tests;

class GlueSpec extends BaseInkTests {
  GlueSpec(Future<Story> Function(String) load,
      {required TestFunction test, required GroupFunction group})
      : super(
          spec: 'Glue',
          createStory: load,
          groupFunction: group,
          testFunction: test,
        );

  late Story story;

  @override
  void tests() {
    setUp(() async {
      story = await createStory(jsonEncode($tests.json));
    });
    it('should go through a tunnel', () {
      story.ChoosePathString('flow_control.tunnel_call');
      expect(story.Continue(), 'tunnel end\n');
      expect(story.canContinue, false);
    });

    it('should follow threads', () {
      story.ChoosePathString('flow_control.thread');
      expect(story.Continue(), 'thread start\n');
      expect(story.Continue(), 'threaded text\n');
      expect(story.Continue(), 'thread end\n');

      expect(story.canContinue, false);
      expect(story.currentChoices.length, 2);
      expect(story.currentChoices[0].text, 'first threaded choice');
      expect(story.currentChoices[1].text, 'second threaded choice');

      story.ChooseChoiceIndex(0);
      expect(story.Continue(), 'first threaded choice\n');
      expect(story.canContinue, false);
    });
  }
}
