import 'dart:convert';

import 'package:ink_runtime/ink.dart';
import '../../base_ink_tests.dart';
import 'package:test/test.dart';

import '../inkfiles/compiled/inkjs/tests.ink.json.dart' as $tests;

class GameQueriesSpec extends BaseInkTests {
  GameQueriesSpec(Future<Story> Function(String) load,
      {required TestFunction test, required GroupFunction group})
      : super(
          spec: 'Game Queries',
          createStory: load,
          groupFunction: group,
          testFunction: test,
        );

  late Story story;

  @override
  void tests() {
    setUp(() async {
      story = await createStory(jsonEncode($tests.json));
    });
    it('should reuturn a choice count', () {
      story.ChoosePathString('game_queries.choicecount');
      story.Continue();

      expect(story.currentChoices.length, 1);
      expect(story.currentChoices[0].text, 'count 0');

      story.ChooseChoiceIndex(0);
      story.Continue();

      expect(story.currentChoices.length, 2);
      expect(story.currentChoices[1].text, 'count 1');

      story.ChooseChoiceIndex(0);
      story.Continue();

      expect(story.currentChoices.length, 3);
      expect(story.currentChoices[2].text, 'count 2');

      story.ChooseChoiceIndex(0);
      story.Continue();

      expect(story.currentChoices.length, 4);
      expect(story.currentChoices[1].text, 'count 1');
      expect(story.currentChoices[3].text, 'count 3');
    });

    it('should return a turn since count', () {
      story.ChoosePathString('game_queries.turnssince_before');
      expect(story.Continue(), '-1\n');
      expect(story.Continue(), '0\n');

      expect(story.currentChoices.length, 1);
      story.ChooseChoiceIndex(0);
      expect(story.Continue(), '1\n');

      expect(story.currentChoices.length, 1);
      story.ChooseChoiceIndex(0);
      expect(story.Continue(), '2\n');
    });
  }
}
