import 'dart:convert';

import 'package:ink_runtime/ink.dart';
import '../../base_ink_tests.dart';
import 'package:test/test.dart';

import '../inkfiles/compiled/inkjs/tests.ink.json.dart' as $tests;

class LogicSpec extends BaseInkTests {
  LogicSpec(Future<Story> Function(String) load,
      {required TestFunction test, required GroupFunction group})
      : super(
          spec: 'Logic',
          createStory: load,
          groupFunction: group,
          testFunction: test,
        );

  late Story story;

  @override
  void tests() {
    setUp(() async {
      story = await createStory(jsonEncode($tests.json));
      story.allowExternalFunctionFallbacks = true;
    });
    it('should define variables', () {
      story.ChoosePathString('logic.vardef');

      expect(story.Continue(), 'variables defined: Emilia 521 52.1\n');
    });

    it('should cast variables', () {
      story.ChoosePathString('logic.casts');
      expect(story.Continue(), '521.5\n');
      expect(story.Continue(), '521hello\n');
      expect(story.Continue(), 'float var is truthy\n');
      expect(story.Continue(), '52.1hello\n');
      expect(story.Continue(), 'string var is truthy\n');
    });

    it('should perform mathematical operations', () {
      story.ChoosePathString('logic.math');

      expect(story.Continue(), '2\n');
      expect(story.Continue(), '0\n');
      expect(story.Continue(), '-5\n');
      expect(story.Continue(), '2\n');
      expect(story.Continue(), '5\n');
      expect(story.Continue(), '1\n');

      expect(story.Continue(), 'int truthy equal\n');
      expect(story.Continue(), 'int falsy equal\n');

      expect(story.Continue(), 'int truthy greater\n');
      expect(story.Continue(), 'int falsy greater\n');

      expect(story.Continue(), 'int truthy lesser\n');
      expect(story.Continue(), 'int falsy lesser\n');

      expect(story.Continue(), 'int truthy greater or equal\n');
      expect(story.Continue(), 'int falsy greater or equal\n');

      expect(story.Continue(), 'int truthy lesser or equal\n');
      expect(story.Continue(), 'int falsy lesser or equal\n');

      expect(story.Continue(), 'int truthy not equal\n');
      expect(story.Continue(), 'int falsy not equal\n');

      expect(story.Continue(), 'int truthy not\n');
      expect(story.Continue(), 'int falsy not\n');

      expect(story.Continue(), 'int truthy and\n');
      expect(story.Continue(), 'int falsy and\n');

      expect(story.Continue(), 'int truthy or\n');
      expect(story.Continue(), 'int falsy or\n');

      expect(story.Continue(), 2.6);
      expect(story.Continue(), 0);
      expect(story.Continue(), -5.2);
      expect(story.Continue(), 3.6);
      expect(story.Continue(), 4.2);
      expect(story.Continue(), 1.5);

      expect(story.Continue(), 'float truthy equal\n');
      expect(story.Continue(), 'float falsy equal\n');

      expect(story.Continue(), 'float truthy greater\n');
      expect(story.Continue(), 'float falsy greater\n');

      expect(story.Continue(), 'float truthy lesser\n');
      expect(story.Continue(), 'float falsy lesser\n');

      expect(story.Continue(), 'float truthy greater or equal\n');
      expect(story.Continue(), 'float falsy greater or equal\n');

      expect(story.Continue(), 'float truthy lesser or equal\n');
      expect(story.Continue(), 'float falsy lesser or equal\n');

      expect(story.Continue(), 'float truthy not equal\n');
      expect(story.Continue(), 'float falsy not equal\n');

      expect(story.Continue(), 'float falsy not\n');

      expect(story.Continue(), 'float truthy and\n');
      expect(story.Continue(), 'float falsy and\n');

      expect(story.Continue(), 'float truthy or\n');
      expect(story.Continue(), 'float falsy or\n');

      expect(story.Continue(), 'truthy string equal\n');
      expect(story.Continue(), 'falsy string equal\n');
      expect(story.Continue(), 'truthy string not equal\n');
      expect(story.Continue(), 'falsy string not equal\n');
      expect(story.Continue(), 'truthy divert equal\n');
      expect(story.Continue(), 'falsy divert equal\n');
    });

    it('should perform if/else tests', () {
      story.ChoosePathString('logic.ifelse');
      expect(story.Continue(), 'if text\n');
      expect(story.Continue(), 'else text\n');
      expect(story.Continue(), 'elseif text\n');
    });

    it('should support params for stitches', () {
      story.ChoosePathString('logic.stitch_param');
      expect(story.Continue(), 'Called with param\n');
    });

    it('should define constants', () {
      story.ChoosePathString('logic.constants');
      expect(story.Continue(), 'constants defined: Emilia 521 52.1\n');
    });

    it('should call ink functions', () {
      story.ChoosePathString('logic.simple_functions');

      expect(story.Continue(), 'returned\n');
      expect(story.Continue(), 'function called\n');
      expect(story.Continue(), 'nested function called\n');
      expect(
          story.Continue(), 'Function called inline and returned something\n');
    });

    it('should call ink functions', () {
      story.ChoosePathString('logic.param_functions');

      expect(story.variablesState['fnParamA'], 'a');
      expect(story.variablesState['fnParamB'], 'b');

      expect(story.Continue(), 'was a\n');
      expect(story.variablesState['fnParamA'], 'a');
      expect(story.variablesState['fnParamB'], 'b');

      expect(story.Continue(), 'was a\n');
      expect(story.variablesState['fnParamA'], 'was a');
      expect(story.variablesState['fnParamB'], 'was b');

      expect(story.canContinue, false);
    });

    it('should call ink functions', () {
      story.ChoosePathString('logic.void_function');
      story.Continue();

      expect(story.canContinue, false);
    });

    it('should generate random numbers', () {
      story.ChoosePathString('logic.random');

      expect(story.Continue(), '15\n');
      expect(story.Continue(), '-24\n');
    });
  }
}
