import 'dart:convert';

import 'package:ink_runtime/ink.dart';
import '../../base_ink_tests.dart';
import 'package:test/test.dart';

import '../inkfiles/compiled/inkjs/tests.ink.json.dart' as $tests;

class SimpleListsSpec extends BaseInkTests {
  SimpleListsSpec(Future<Story> Function(String) load,
      {required TestFunction test, required GroupFunction group})
      : super(
          spec: 'Simple Lists',
          createStory: load,
          groupFunction: group,
          testFunction: test,
        );

  late Story story;

  @override
  void tests() {
    setUp(() async {
      story = await createStory(jsonEncode($tests.json));
      story.allowExternalFunctionFallbacks = true;
    });
    it('should go through a sequence', () {
      story.ChoosePathString('simple_lists.sequence');
      expect(story.Continue(), 'one\n');

      story.ChoosePathString('simple_lists.sequence');
      expect(story.Continue(), 'two\n');

      story.ChoosePathString('simple_lists.sequence');
      expect(story.Continue(), 'three\n');

      story.ChoosePathString('simple_lists.sequence');
      expect(story.Continue(), 'final\n');

      story.ChoosePathString('simple_lists.sequence');
      expect(story.Continue(), 'final\n');
    });

    it('should go through a cycle', () {
      var results = ['one\n', 'two\n', 'three\n'];

      for (var i = 0; i < 10; ++i) {
        story.ChoosePathString('simple_lists.cycle');
        expect(story.Continue(), results[i % 3]);
      }
    });

    it('should go through a list once', () {
      story.ChoosePathString('simple_lists.once');
      expect(story.Continue(), 'one\n');

      story.ChoosePathString('simple_lists.once');
      expect(story.Continue(), 'two\n');

      story.ChoosePathString('simple_lists.once');
      expect(story.Continue(), 'three\n');

      story.ChoosePathString('simple_lists.once');
      expect(story.Continue(), '');
    });

    it('should go through a shuffle', () {
      var results = ['heads\n', 'tails\n'];

      for (var i = 0; i < 40; ++i) {
        story.ChoosePathString('simple_lists.shuffle');
        expect(results, contains(story.Continue()));
      }
    });

    it('should handle blank elements', () {
      for (var i = 0; i < 3; ++i) {
        story.ChoosePathString('simple_lists.blanks');
        expect(story.Continue(), '');
      }

      story.ChoosePathString('simple_lists.blanks');
      expect(story.Continue(), 'end\n');
    });
  }
}
