import 'dart:convert';

import 'package:ink_runtime/ink.dart';
import '../../base_ink_tests.dart';
import 'package:test/test.dart';

import '../inkfiles/compiled/inkjs/tests.ink.json.dart' as $tests;

class InkjsTagsSpec extends BaseInkTests {
  InkjsTagsSpec(Future<Story> Function(String) load,
      {required TestFunction test, required GroupFunction group})
      : super(
          spec: 'Tags',
          createStory: load,
          groupFunction: group,
          testFunction: test,
        );

  late Story story;

  @override
  void tests() {
    setUp(() async {
      story = await createStory(jsonEncode($tests.json));
      story.allowExternalFunctionFallbacks = true;
    });
    it('should find global tags', () {
      var tags = story.globalTags;

      expect(tags.length, 1);
      expect(tags[0], 'global tag');
    });

    it('should find knot level tags', () {
      var tags = story.TagsForContentAtPath('tags');

      expect(tags.length, 1);
      expect(tags[0], 'knot tag');
    });

    it('should find line by line tags', () {
      story.ChoosePathString('tags.line_by_Line');
      story.Continue();

      var tags = story.currentTags;

      expect(tags.length, 1);
      expect(tags[0], 'a tag');

      story.Continue();
      tags = story.currentTags;

      expect(tags.length, 2);
      expect(tags[0], 'tag1');
      expect(tags[1], 'tag2');

      story.Continue();
      tags = story.currentTags;

      expect(tags.length, 2);
      expect(tags[0], 'tag above');
      expect(tags[1], 'tag after');
    });

    it('should find tags on choice points', () {
      story.ChoosePathString('tags.choice');
      story.Continue();

      expect(story.currentChoices.length, 1);
      expect(story.currentChoices[0].text, 'a choice');
      expect(story.currentTags.length, 0);

      story.ChooseChoiceIndex(0);

      expect(story.Continue(), 'a choice\n');
      expect(story.currentTags.length, 1);
      expect(story.currentTags[0], 'a tag');
    });

    it('should handle tag edge cases', () {
      story.ChoosePathString('tags.weird');
      story.Continue();

      var tags = story.currentTags;

      expect(tags.length, 5);
      expect(tags[0], 'space around');
      expect(tags[1], '');
      expect(tags[2], '');
      expect(tags[3], '');
      expect(tags[4], '0');
    });
  }
}
