import 'package:ink_runtime/ink.dart';
import 'package:test/test.dart';

import '../../runtime_test.dart';
import '../base_ink_tests.dart';

class TagsSpec extends BaseInkTests {
  TagsSpec(Future<Story> Function(String) load,
      {required TestFunction test, required GroupFunction group})
      : super(
          spec: 'Tags',
          createStory: load,
          groupFunction: group,
          testFunction: test,
        );

  @override
  void tests() {
    testFunction('tests string constants', () async {
      var story = await createStory(await loadFromUrl(
          'https://raw.githubusercontent.com/y-lohse/inkjs/v2.0.0/src/tests/inkfiles/compiled/tags/tags.ink.json'));

      var globalTags = ['author: Joe', 'title: My Great Story'];
      var knotTags = ['knot tag'];
      var knotTagsWhenContinuedTwiceTags = ['end of knot tag'];
      var stitchTags = ['stitch tag'];

      expect(story.globalTags, globalTags);
      expect(story.Continue(), 'This is the content\n');
      expect(story.currentTags, globalTags);

      expect(story.TagsForContentAtPath('knot'), knotTags);
      expect(story.TagsForContentAtPath('knot.stitch'), stitchTags);

      story.ChoosePathString('knot');
      expect(story.Continue(), 'Knot content\n');
      expect(story.currentTags, knotTags);
      expect(story.Continue(), '');
      expect(story.currentTags, knotTagsWhenContinuedTwiceTags);
    });

    testFunction('tests tag on choice', () async {
      var story = await createStory(await loadFromUrl(
          'https://raw.githubusercontent.com/y-lohse/inkjs/v2.0.0/src/tests/inkfiles/compiled/tags/tags_on_choice.ink.json'));
      story.Continue();
      story.ChooseChoiceIndex(0);

      var txt = story.Continue();
      var tags = story.currentTags;

      expect(txt, 'Hello');
      expect(tags.length, 1);
      expect(tags[0], 'hey');
    });
  }
}
